#! /usr/bin/python

"""//Web service to pay a batch order//
 
- > When an order have to be closed (for instance, when the batch processing is finished), the pay operation must be called to acknowledge end of resources consumption.

- File: pay.py 
  
- Arguments - Using the HTTP GET method:
 - batch_id -- The batch identifier, as a string
 - > If some arguments are missing, a web form will be displayed.
 
- Displays a Web page containing:
 - ``<div class="qos">True</div>``if the pay operation is correctly completed
  
- Notes: 
 - A database configuration file named "db.cfg" must be present in the directory.

"""
import sys
import os
sys.path.append(os.path.join(sys.path[0],".."))
os.chdir(sys.path[0])
from tools import db_func
from tools import cgi_func
from tools import log_func
log=log_func.get_spequlos_log(__file__)




def pay(conn,batch_id):
    """Function implementing the  order operation

    Arguments:
    conn -- The CreditSystem's DB connection, as a MySQL.connections.Connection object
    batch_id -- The batch identifier, as a string
    user_id --- The user identifier, as an integer
    credit -- The number of credits to allocate, as a positive integer
    
    The function returns True if the order is accepted, or False otherwise

    """
    if not isinstance(conn,db_func.db_connection) or conn.open==0:
        log.error("The conn argument must be an open connection to the database !")
        return None       
    if not isinstance(batch_id,str):
        log.error("The batch_id argument must be a string !")
        return None        
            
    c=conn.cursor()

    log.info("Retrieving order for batch identifier %s..." % batch_id)
    try:
        c.execute("SELECT credit_balance,credit_funded,user_id,cloud_id FROM QoSOrder WHERE batch_id='%s'" % batch_id)
    except db_func.DBError as e:
        log.error("Error during the SQL command execution !")
        log.error(e)
        return None  
    res = c.fetchall()
    
    if len(res)==0:
        log.error("No such order associated to this batch identifier in the database")
        return None
    elif len(res)>1:
        log.error("Several orders associated with batch identifier %d in the database ! 'batch_id' should be QoSOrder PRIMARY KEY !" % batch_id)
        return None

    try:
        order_credit_balance = int(res[0][0])
        if order_credit_balance<0:
            raise ValueError("Negative credit_balance !")
    except ValueError as e:
        log.error("The 'credit_balance' in QoSOrder table must be a positive or zero integer !")
        log.error(e)  
        return None              
    try:
        order_credit_funded = int(res[0][1])
        if order_credit_funded<0:
            raise ValueError("Negative credit_funded !")
    except ValueError as e:
        log.error("The 'credit_funded' in QoSOrder table must be a positive or zero integer !")
        log.error(e)  
        return None      
    try:
        order_user_id = int(res[0][2])
    except ValueError as e:
        log.error("The 'user_id' in QoSOrder table must be integer !")
        log.error(e)  
        return None      
    try:
        order_cloud_id = int(res[0][3])
    except ValueError as e:
        log.error("The 'cloud_id' in QoSOrder table must be integer !")
        log.error(e)  
        return None  
                            
    order_credit_spent = order_credit_funded - order_credit_balance
    
    log.info("Order found, credit balance is %d, credit funded was %d (spent: %d), user identifier is %d, cloud identifier is %d" \
            % (order_credit_balance,order_credit_funded,order_credit_spent,order_user_id,order_cloud_id))

    log.info("Retrieving batch user...")
    try:
        c.execute("SELECT credit_spent,institution_id FROM QoSUser WHERE user_id=%d" % order_user_id)
    except db_func.DBError as e:
        log.error("Error during the SQL command execution !")
        log.error(e)
        return None  
    res = c.fetchall()
    
    if len(res)==0:
        log.error("Error: no such user associated to the order in the database")
        return None
    elif len(res)>1:
        log.error("Error: users associated to this user identifier in the database ! 'user_id' should be QoSUser PRIMARY KEY !"% order_user_id)
        return None

    try:
        user_credit_spent = int(res[0][0])
        if user_credit_spent<0:
            raise ValueError("Negative credit_spent !")
    except ValueError as e:
        log.error("The 'credit_spent' in QoSUser table must be a positive or zero integer !")
        log.error(e)  
        return None      
    try:
        user_institution_id = int(res[0][1])
    except ValueError as e:
        log.error("The 'institution_id' in QoSUser table must be integer !")
        log.error(e)  
        return None                

    log.info("Batch user found, updating credit spent (was %d)..." % user_credit_spent)
    try:
        c.execute("UPDATE QoSUser SET credit_spent=%d WHERE user_id=%d" % (user_credit_spent+order_credit_spent,order_user_id))
    except db_func.DBError as e:
        log.error("Error during the SQL command execution !")
        log.error(e)
        return None      
    log.info("Done! New credit spent is %d" % (user_credit_spent + order_credit_spent))

    log.info("Acknowledging payment to institution account...")
    try:
        c.execute("SELECT credit_balance,credit_spent FROM Institution WHERE institution_id=%d" % user_institution_id)
    except db_func.DBError as e:
        log.error("Error during the SQL command execution !")
        log.error(e)
        return None  
    res = c.fetchall()
    
    if len(res)==0:
        log.error("No such institution associated to this user %d" % user_institution_id)
        return None
    elif len(res)>1:
        log.error("Several institution with this identifier in the database ! 'institution_id' should be Institution PRIMARY KEY !"% user_institution_id)
        return None
    
    try:
        institution_credit_balance=res[0][0]
        if institution_credit_balance<0:
            raise ValueError("Negative credit_balance !")
    except ValueError as e:
        log.error("The 'credit_balance' in Institution table must be a positive or zero integer !")
        log.error(e)  
        return None      
    try:
        institution_credit_spent=res[0][1]
        if institution_credit_spent<0:
            raise ValueError("Negative credit_spent !")
    except ValueError as e:
        log.error("The 'credit_spent' in Institution table must be a positive or zero integer !")
        log.error(e)  
        return None      
    
    log.info("Institution found, its credit balance was %d and %d credits was already spent" \
            % (institution_credit_balance,institution_credit_spent))

    log.info("Updating credit spent")
    try:
        c.execute("UPDATE Institution SET credit_spent=%d WHERE institution_id=%d" % (institution_credit_spent+order_credit_spent,user_institution_id))
    except db_func.DBError as e:
        log.error("Error during the SQL command execution !")
        log.error(e)
        return None  
    log.info("Done! New credit spent is %d" % (institution_credit_spent+order_credit_spent))

    if order_credit_balance>0:
        log.info("Some credits have not been spended, refunding the institution balance...")
        try:
            c.execute("UPDATE Institution SET credit_balance=%d WHERE institution_id=%d" \
                % (institution_credit_balance+order_credit_balance,user_institution_id))
        except db_func.DBError as e:
            log.error("Error during the SQL command execution !")
            log.error(e)
            return None          
        log.info("Done! New institution balance is %d" % (institution_credit_balance+order_credit_balance))
    else:
        log.info("All credits have been spent, nothing to refund")

    log.info("Retrieving cloud associated with the order...")
    try:
        c.execute("SELECT credit_spent FROM Cloud WHERE cloud_id=%d" % order_cloud_id)
    except db_func.DBError as e:
        log.error("Error during the SQL command execution !")
        log.error(e)
        return None  
    res = c.fetchall()
    
    if len(res)==0:
        log.error("No such cloud associated to this identifier %d" % order_cloud_id)
        return None
    elif len(res)>1:
        log.error("Several clouds with this identifier in the database ! "% order_cloud_id)
        return None
    
    try:
        cloud_credit_spent = int(res[0][0])
        if cloud_credit_spent<0:
            raise ValueError("Negative credit_spent !")
    except ValueError as e:
        log.error("The 'credit_spent' in Cloud table must be a positive or zero integer !")
        log.error(e)  
        return None      

    log.info("Cloud found, %d credits was already spent, updating..." % cloud_credit_spent)

    try:
        c.execute("UPDATE Cloud SET credit_spent=%d WHERE cloud_id=%d" % (cloud_credit_spent+order_credit_spent,order_cloud_id))
    except db_func.DBError as e:
        log.error("Error during the SQL command execution !")
        log.error(e)
        return None  
    log.info("Done! New credit spent is %d" % (cloud_credit_spent+order_credit_spent))

    log.info("Removing the order from the database...")
    try:
        c.execute("DELETE FROM QoSOrder WHERE batch_id='%s'" % batch_id)
    except db_func.DBError as e:
        log.error("Error during the SQL command execution !")
        log.error(e)
        return None  
    log.info("Done!")

    return True

if __name__=="__main__":
    cgi_func.print_header()

    if cgi_func.get_form_value('batch_id') is None:
        cgi_func.print_form(['batch_id'])

    else:
        batch_id = cgi_func.get_form_value('batch_id')
        conn=db_func.connect(db_func.read_db_config("db.cfg"))
        cgi_func.print_result( pay(conn,batch_id))
        db_func.disconnect(conn)
        
    cgi_func.print_footer()

