#! /usr/bin/python

"""//Module for database related operation//

- File: db_func.py

- Database used: MySQL

"""

import MySQLdb
import conf_func
import log_func
log=log_func.get_spequlos_log(__file__)

db_connection=MySQLdb._mysql.connection
DBError=MySQLdb.ProgrammingError

def read_db_config(filename):
    """//Read a database configuration file//

    - Arguments:
     - filename --- The configuration file name, as a string

    - The function returns a dictionary containing the keys: "host", "db", "user", "passwd", with the according values as read in the file.
         
    """
    params=conf_func.read_config(filename)
    if not "host" in params:
        log.error("Please inform the host option in the db configuration file ("+filename+")")
        raise Exception("No host information in database configuration file: %s" %filename)
    if not "db" in params:
        log.error("Please inform the db option in the db configuration file ("+filename+")")
        raise Exception("No database name information in database configuration file: %s" %filename)
    if not "user" in params:
        log.error("Please inform the user option in the db configuration file ("+filename+")")
        raise Exception("No user information in database configuration file: %s" %filename)
    if not "passwd" in params:
        log.error("Please inform the passwd option in the db configuration file ("+filename+")")
        raise Exception("No password information in database configuration file: %s" %filename)
    return params


def connect(params):
    """//Connect to a MySQL database//

    - Arguments:
     - params --- The database configuration parameters, as a dictionary. It must contain the right values for database access in the keys: "host", "db", "user", "passwd".

    - The function returns a MySQLdb.connections.Connection instance on the database connection
    
    """
    log.info("Connecting to database...")
    try:
        if "port" in params:
            try:
                port=int(params["port"])
            except Exception as e:
                log.error("Error when reading DB port in parameters")
                log.error(e)
                raise e
            else:
                conn = MySQLdb.connect(host=params["host"],db=params["db"],user=params["user"],passwd=params["passwd"],port=port)
        else:
            conn = MySQLdb.connect(host=params["host"],db=params["db"],user=params["user"],passwd=params["passwd"])
    except Exception as e:
        log.error("Error when connecting to database")
        log.error(e)        
        raise e
    log.info("Done !")
    return conn


def disconnect(conn):
    """//Disconnect from a connected MySQL database//

    - Arguments:
     - conn --- The database connection, as a MySQLdb.connections.Connection
        
    """    
    log.info("Disconnecting from database...")
    conn.close()
    log.info("Done !")


class SpeQuloS_DB:
    
    def __init__(self,configuration_file="db.cfg"):
        self.conn=None
        self.cur=None
        self.conn=connect(read_db_config(configuration_file))
        self.cur=self.conn.cursor()
            
    def __del__(self):
        if self.cur:
            self.cur.close()
        if self.conn:
            self.conn.close()

    def sql_exec(self,sql_command):
        try:
            #print sql_command
            self.cur.execute(sql_command)
        except DBError as e:
            log.error("Error during the SQL command execution !")
            log.error("Command was: %s" % sql_command)
            log.error(e)
            raise e
            
    def sql_select(self,table,columns,condition=""):
        try:
            if not isinstance(table,str):
                try:
                    for s in table:
                        if not isinstance(s,str):
                            raise Exception("No valid SQL SELECT arguments")                       
                except Exception as e:
                    raise Exception("No valid SQL SELECT arguments: "+e)                
            if not isinstance(condition,str):
                raise Exception("No valid SQL SELECT arguments")                
            for s in columns:
                if not isinstance(s,str):
                    raise Exception("No valid SQL SELECT arguments")                
                                    
            col_str=""
            for c in columns:
                col_str=col_str+c+","
            col_str=col_str[0:-1]
            
            if isinstance(table,str):            
                sql="SELECT %s FROM %s" % (col_str,table)
            else:
                sql="SELECT %s FROM %s" % (col_str,table[0])
                for t in table[1:]:
                    sql+=" NATURAL JOIN %s" % t
                    
            if condition!="":
                sql+=" WHERE %s" % condition

            self.sql_exec(sql)
            res=self.cur.fetchall()
            
            dict_list=[]
            for e in res:
                d={}
                i=0
                for c in columns:
                    d[c]=e[i]
                    i+=1
                dict_list.append(d)
            return dict_list
        except Exception as e:                
            log.error("Error during the SQL SELECT command execution !")
            log.error(e)
            raise e            
             
    def sql_delete(self,table,condition=""):
        try:
            if not isinstance(table,str):
                raise Exception("No valid SQL DELETE arguments")                
            if not isinstance(condition,str):
                raise Exception("No valid SQL DELETE arguments")                
            
            sql="DELETE FROM %s" % table
            if condition!="":
                sql+=" WHERE %s" % condition

            self.sql_exec(sql)
            
        except Exception as e:                
            log.error("Error during the SQL DELETE command execution !")
            log.error(e)
            raise e            

    def sql_insert(self,table,columns,values):
        #try:
            if not isinstance(table,str):
                raise Exception("No valid SQL INSERT arguments")                
            for s in columns:
                if not isinstance(s,str):
                    raise Exception("No valid SQL INSERT arguments")                
            iter(values)
            if len(values)!=len(columns):
                raise Exception("No valid SQL INSERT arguments")                
            
            col_str="("
            for c in columns:
                col_str=col_str+c+","
            col_str=col_str[0:-1]
            col_str+=")"

            val_str="("
            for v in values:
                if isinstance(v,str) and v!="NULL":
                    val_str=val_str+"'"+v+"',"
                else:
                    val_str=val_str+"%s," % v
            val_str=val_str[0:-1]
            val_str+=")"
            
            sql="INSERT INTO %s %s VALUES %s" % (table,col_str,val_str)
            
            self.sql_exec(sql)
            
        #except Exception as e:                
        #    log.error("Error during the SQL INSERT command execution !")
        #    log.error(e)
        #    raise e            


    def sql_update(self,table,columns,values,condition=""):
        try:
            if not isinstance(table,str):
                raise Exception("No valid SQL UPDATE arguments")                
            if not isinstance(condition,str):
                raise Exception("No valid SQL UPDATE arguments")                
            for s in columns:
                if not isinstance(s,str):
                    raise Exception("No valid SQL UPDATE arguments")                
            iter(values)
            if len(values)!=len(columns):
                raise Exception("No valid SQL UPDATE arguments")                
                                    
            sql="UPDATE %s SET " % table
            for i in range(len(columns)):
                if i !=0:
                    sql+=" , "
                if isinstance(values[i],str) and values[i]!="NULL":
                    sql+="%s='%s'" % (columns[i],values[i])                
                else:
                    sql+="%s=%s" % (columns[i],values[i])                
            if condition!="":
                sql+=" WHERE %s" % condition

            self.sql_exec(sql)
            
        except Exception as e:                
            log.error("Error during the SQL UPDATE command execution !")
            log.error(e)
            raise e            


            
if __name__=="__main__":
    import sys
    db = SpeQuloS_DB(sys.argv[1])
    db.sql_exec("DROP TABLE IF EXISTS sqltable")
    db.sql_exec("DROP TABLE IF EXISTS sqltable2")
    db.sql_exec("CREATE TABLE sqltable ( a int, b int, s text )")
    db.sql_insert("sqltable",('a','b','s'),(1,2,'toto'))
    db.sql_insert("sqltable",('a','b'),(3,4))
    for l in db.sql_select("sqltable",('a','b','s')):
        for v in l:
            print v,l[v],type(l[v])
    for l in db.sql_select("sqltable",('a','b','s'),"s!='NULL'"):
        print l
    db.sql_update("sqltable",('a','b'),(7,8),"s IS NULL")
    for l in db.sql_select("sqltable",('a','b','s')):
        print l
    db.sql_exec("CREATE TABLE sqltable2 ( a int, c int, t text )")    
    db.sql_insert("sqltable2",('a','c','t'),(1,1000,'titi'))    
    for l in db.sql_select(("sqltable","sqltable2"),('a','b','c','s','t')):
        print l
    db.sql_delete("sqltable","s IS NULL")
    for l in db.sql_select("sqltable",('a','b','s')):
        print l
    db.sql_exec("DROP TABLE sqltable")
    db.sql_exec("DROP TABLE IF EXISTS sqltable2")
            
